@@ -1,22 +1,24 @@
+# -*- coding: utf-8 -*-
+
 """KidWriter
 
 Write Python source code from XML.
 
 """
 
-__revision__ = "$Rev: 470 $"
-__date__ = "$Date: 2007-01-12 20:25:43 -0500 (Fri, 12 Jan 2007) $"
+__revision__ = "$Rev: 496 $"
+__date__ = "$Date: 2007-07-15 18:14:35 -0400 (Sun, 15 Jul 2007) $"
 __author__ = "Ryan Tomayko (rtomayko@gmail.com)"
 __copyright__ = "Copyright 2004-2005, Ryan Tomayko"
 __license__ = "MIT <http://www.opensource.org/licenses/mit-license.php>"
 
 import sys, re
+from os.path import splitext
 from traceback import extract_tb, format_exception_only
 
-from kid import __version__
+from kid import __version__, Namespace
 from kid.parser import document, START, END, TEXT, XML_DECL, DOCTYPE, LOCATION
 from kid.element import namespaces, Comment, ProcessingInstruction
-from kid import Namespace
 
 __all__ = ['KID_XMLNS', 'KID_PREFIX', 'kidns', 'raise_template_error']
 
@@ -79,6 +81,20 @@ def error_location(filename, encoding=None, entity_map=None, lineno=None):
         except Exception:
             pass
 
+def TemplateExceptionError(error, add_message):
+    """Get exception with additional error message."""
+    Error = error.__class__
+    class TemplateExceptionError(Error):
+        def __init__(self):
+            for arg in dir(error):
+                if not arg.startswith('_'):
+                    setattr(self, arg, getattr(error, arg))
+        def __str__(self):
+            return str(error) + '\n' + add_message
+    TemplateExceptionError.__name__ = Error.__name__
+    TemplateExceptionError.__module__ = Error.__module__
+    return TemplateExceptionError()
+
 def raise_template_error(module=None, filename=None, encoding=None):
     """Raise template error along with additional context information.
 
@@ -89,54 +105,57 @@ def raise_template_error(module=None, filename=None, encoding=None):
 
     """
     if module and not (filename and encoding):
-        try:
+        if module in sys.modules:
             mod = sys.modules[module]
-            encoding = mod.encoding
-            filename = mod.kid_file
-        except Exception:
-            pass
+            if hasattr(mod, 'encoding'):
+                encoding = mod.encoding
+            if hasattr(mod, 'kid_file'):
+                filename = mod.kid_file
+    if not filename or filename == '<string>':
+        raise
     if not encoding:
         encoding = 'utf-8'
-    if filename and filename != '<string>':
-        exc_type, exc_value = sys.exc_info()[:2]
-        lineno = getattr(exc_value, 'lineno',
-            extract_tb(sys.exc_info()[2])[-1][1])
-        if lineno:
-            location = error_location(filename, encoding, lineno=lineno)
-            if location:
-                (start_line, start_col), (end_line, end_col) = location
-                if start_line > end_line:
-                    location = None
+    py_file = splitext(filename)[0] + '.py'
+    exc_type, exc_value = sys.exc_info()[:2]
+    if exc_type == SyntaxError:
+        tb = [(py_file, exc_value.lineno)]
+    else:
+        tb = extract_tb(sys.exc_info()[2])
+        tb.reverse()
+    for t in tb:
+        if py_file != t[0] or not t[1]:
+            continue
+        location = error_location(filename, encoding, lineno=t[1])
+        if not location:
+            continue
+        (start_line, start_col), (end_line, end_col) = location
+        if start_line > end_line:
+            continue
+        s = []
+        if not end_col and end_line > start_line:
+            end_line -= 1
+            end_col = -1
+        if start_line == end_line:
+            s.append('on line %d' % start_line)
+            if start_col == end_col:
+                s.append(', column %d' % start_col)
+            elif start_col:
+                if end_col > start_col:
+                    s.append(' between columns %d and %d'
+                        % (start_col, end_col))
+                else:
+                    s.append(' after column %d' % start_col)
+            elif end_col > 0:
+                s.append(' before column %d' % end_col)
         else:
-            location = None
-        if location:
-            s = []
-            if not end_col and end_line > start_line:
-                end_line -= 1
-                end_col = -1
-            if start_line == end_line:
-                s.append('on line %d' % start_line)
-                if start_col == end_col:
-                    s.append(', column %d' % start_col)
-                elif start_col:
-                    if end_col > start_col:
-                        s.append(' between columns %d and %d'
-                            % (start_col, end_col))
-                    else:
-                        s.append(' after column %d' % start_col)
-                elif end_col > 0:
-                    s.append(' before column %d' % end_col)
-            else:
-                s.append('between line %d' % start_line)
-                if start_col:
-                    s.append(', column %d' % start_col)
-                s.append(' and line %d' % end_line)
-                if end_col > 0:
-                    s.append(', column %d' % end_col)
-            s = ''.join(s)
-        else:
-            s = None
+            s.append('between line %d' % start_line)
+            if start_col:
+                s.append(', column %d' % start_col)
+            s.append(' and line %d' % end_line)
+            if end_col > 0:
+                s.append(', column %d' % end_col)
         if s:
+            s = ''.join(s)
             try:
                 start_line -= 1
                 end_line -= 1
@@ -171,24 +190,28 @@ def raise_template_error(module=None, filename=None, encoding=None):
                                 error_text[2] = text
                     if line >= end_line:
                         break
-                if error_line:
-                    if len(error_line) == 2:
-                        if error_line[1] - error_line[0] > 1:
-                            error_text.insert(1, '...')
-                    elif len(error_line) == 3:
-                        if error_line[2] - error_line[0] > 2:
-                            error_text[1] = '...'
-                    s = [s + ':'] + error_text
+                if not error_line:
+                    raise LookupError, 'error line not found'
+                if len(error_line) == 2:
+                    if error_line[1] - error_line[0] > 1:
+                        error_text.insert(1, '...')
+                elif len(error_line) == 3:
+                    if error_line[2] - error_line[0] > 2:
+                        error_text[1] = '...'
+                s = [s + ':'] + error_text
             except Exception, e:
                 s = [s, '(cannot acquire source text: %s)' % str(e)]
             s.insert(0, 'Error location in template file %r' % filename)
-        else:
-            s = ['Error in code generated from template file %r' % filename]
-        exc_value = ''.join([str(exc_value), '\n'] +
-            format_exception_only(exc_type, exc_value)[:-1]) + '\n'.join(s)
-        raise exc_type, exc_value, sys.exc_info()[2]
+            break
     else:
-        raise
+        s = ['Error in code generated from template file %r' % filename]
+    s = ''.join(format_exception_only(exc_type, exc_value)[:-1]) + '\n'.join(s)
+    if isinstance(exc_type, str):
+        exc_type += '\n' + s
+    else:
+        exc_value = TemplateExceptionError(exc_value, s)
+        exc_type = exc_value.__class__
+    raise exc_type, exc_value, sys.exc_info()[2]
 
 
 class KidWriter(object):
